@@ -1,20 +1,70 @@
-""" Vector Quantization Module
+""" K-means Clustering and Vector Quantization Module
 
-    Provides several routines used in creating a code book from a set of
-    observations and comparing a set of observations to a code book.
+    Provides routines for k-means clustering, generating code books
+    from k-means models, and quantizing vectors by comparing them with
+    centroids in a code book.
 
-    All routines expect an "observation vector" to be stored in each
-    row of the obs matrix.  Similarly the codes are stored row wise
-    in the code book matrix.
+    The k-means algorithm takes as input the number of clusters to
+    generate, k, and a set of observation vectors to cluster.  It
+    returns a set of centroids, one for each of the k clusters.  An
+    observation vector is classified with the cluster number or
+    centroid index of the centroid closest to it.
+
+    A vector v belongs to cluster i if it is closer to centroid i than
+    any other centroids. If v belongs to i, we say centroid i is the
+    dominating centroid of v. Common variants of k-means try to
+    minimize distortion, which is defined as the sum of the distances
+    between each observation vector and its dominating centroid.  Each
+    step of the k-means algorithm refines the choices of centroids to
+    reduce distortion. The change in distortion is often used as a
+    stopping criterion: when the change is lower than a threshold, the
+    k-means algorithm is not making sufficient progress and
+    terminates.
+
+    Since vector quantization is a natural application for k-means,
+    information theory terminology is often used.  The centroid index
+    or cluster index is also referred to as a "code" and the table
+    mapping codes to centroids and vice versa is often referred as a
+    "code book". The result of k-means, a set of centroids, can be
+    used to quantize vectors. Quantization aims to find an encoding of
+    vectors that reduces the expected distortion.
+
+    For example, suppose we wish to compress a 24-bit color image
+    (each pixel is represented by one byte for red, one for blue, and
+    one for green) before sending it over the web.  By using a smaller
+    8-bit encoding, we can reduce the amount of data by two
+    thirds. Ideally, the colors for each of the 256 possible 8-bit
+    encoding values should be chosen to minimize distortion of the
+    color. Running k-means with k=256 generates a code book of 256
+    codes, which fills up all possible 8-bit sequences.  Instead of
+    sending a 3-byte value for each pixel, the 8-bit centroid index
+    (or code word) of the dominating centroid is transmitted. The code
+    book is also sent over the wire so each 8-bit code can be
+    translated back to a 24-bit pixel value representation. If the
+    image of interest was of an ocean, we would expect many 24-bit
+    blues to be represented by 8-bit codes. If it was an image of a
+    human face, more flesh tone colors would be represented in the
+    code book.
+
+    All routines expect obs to be a M by N array where the rows are
+    the observation vectors. The codebook is a k by N array where the
+    i'th row is the centroid of code word i. The observation vectors
+    and centroids have the same feature dimension.
 
     whiten(obs) --
-        Normalize a group of observations on a per feature basis
+        Normalize a group of observations so each feature has unit
+        variance.
     vq(obs,code_book) --
-        Calculate code book membership of obs
+        Calculate code book membership of a set of observation
+        vectors.
     kmeans(obs,k_or_guess,iter=20,thresh=1e-5) --
-        Train a codebook for mimimum distortion using the kmeans algorithm
-    kmeans2
-        Similar to kmeans, but with several initialization methods.
+        Clusters a set of observation vectors. Learns centroids with
+        the k-means algorithm, trying to minimize distortion.  A code
+        book is generated that can be used to quantize vectors.
+    kmeans2 --
+        A different implementation of k-means with more methods for
+        initializing centroids.  Uses maximum number of iterations as
+        opposed to a distortion threshold as its stopping criterion.
 
 """
 __docformat__ = 'restructuredtext'
@@ -22,8 +72,8 @@ __docformat__ = 'restructuredtext'
 __all__ = ['whiten', 'vq', 'kmeans', 'kmeans2']
 
 # TODO:
-#   - implements high level method for running several times kmeans with
-#   different initialialization 
+#   - implements high level method for running several times k-means with
+#   different initialialization
 #   - warning: what happens if different number of clusters ? For now, emit a
 #   warning, but it is not great, because I am not sure it really make sense to
 #   succeed in this case (maybe an exception is better ?)
@@ -33,7 +83,7 @@ from numpy.random import randint
 from numpy import shape, zeros, sqrt, argmin, minimum, array, \
      newaxis, arange, compress, equal, common_type, single, double, take, \
      std, mean
-import numpy as N
+import numpy as np
 
 class ClusterError(Exception):
     pass
@@ -41,15 +91,15 @@ class ClusterError(Exception):
 def whiten(obs):
     """ Normalize a group of observations on a per feature basis.
 
-    Before running kmeans algorithms, it is beneficial to "whiten", or
-    scale, the observation data on a per feature basis.  This is done
-    by dividing each feature by its standard deviation across all
-    observations.
+    Before running k-means, it is beneficial to rescale each feature
+    dimension of the observation set with whitening. Each feature is
+    divided by its standard deviation across all observations to give
+    it unit variance.
 
     :Parameters:
         obs : ndarray
             Each row of the array is an observation.  The
-            columns are the "features" seen during each observation
+            columns are the features seen during each observation.
             ::
 
                       #   f0    f1    f2
@@ -83,23 +133,25 @@ def whiten(obs):
     return obs / std_dev
 
 def vq(obs, code_book):
-    """ Vector Quantization: assign features sets to codes in a code book.
+    """ Vector Quantization: assign codes from a code book to observations.
 
-    Vector quantization determines which code in the code book best represents
-    an observation of a target.  The features of each observation are compared
-    to each code in the book, and assigned the one closest to it.  The
-    observations are contained in the obs array. These features should be
-    "whitened," or nomalized by the standard deviation of all the features
-    before being quantized.  The code book can be created using the kmeans
-    algorithm or something similar.
+    Assigns a code from a code book to each observation. Each
+    observation vector in the M by N obs array is compared with the
+    centroids in the code book and assigned the code of the closest
+    centroid.
+
+    The features in obs should have unit variance, which can be
+    acheived by passing them through the whiten function.  The code
+    book can be created with the k-means algorithm or a different
+    encoding algorithm.
 
     :Parameters:
         obs : ndarray
-            Each row of the array is an observation.  The columns are the
-            "features" seen during each observation The features must be
+            Each row of the NxM array is an observation.  The columns are the
+            "features" seen during each observation. The features must be
             whitened first using the whiten function or something equivalent.
         code_book : ndarray.
-            The code book is usually generated using the kmeans algorithm.
+            The code book is usually generated using the k-means algorithm.
             Each row of the array holds a different code, and the columns are
             the features of the code.
 
@@ -112,15 +164,14 @@ def vq(obs, code_book):
 
     :Returns:
         code : ndarray
-            If obs is a NxM array, then a length N array is returned that holds
-            the selected code book index for each observation.
+            A length N array holding the code book index for each observation.
         dist : ndarray
             The distortion (distance) between the observation and its nearest
-            code
+            code.
 
     Notes
     -----
-    This currently forces 32 bit math precision for speed.  Anyone know
+    This currently forces 32-bit math precision for speed.  Anyone know
     of a situation where this undermines the accuracy of the algorithm?
 
     Examples
@@ -154,7 +205,7 @@ def vq(obs, code_book):
 def py_vq(obs, code_book):
     """ Python version of vq algorithm.
 
-    The algorithm simply computes the euclidian distance between each
+    The algorithm computes the euclidian distance between each
     observation and every frame in the code_book.
 
     :Parameters:
@@ -165,11 +216,11 @@ def py_vq(obs, code_book):
             features (eg columns) than obs.
 
     :Note:
-        This function is slower than the C versions, but it works for
+        This function is slower than the C version but works for
         all input types.  If the inputs have the wrong types for the
         C versions of the function, this one is called as a last resort.
 
-        Its about 20 times slower than the C versions.
+        It is about 20 times slower than the C version.
 
     :Returns:
         code : ndarray
@@ -182,8 +233,8 @@ def py_vq(obs, code_book):
     """
     # n = number of observations
     # d = number of features
-    if N.ndim(obs) == 1:
-        if not N.ndim(obs) == N.ndim(code_book):
+    if np.ndim(obs) == 1:
+        if not np.ndim(obs) == np.ndim(code_book):
             raise ValueError(
                     "Observation and code_book should have the same rank")
         else:
@@ -193,7 +244,7 @@ def py_vq(obs, code_book):
 
     # code books and observations should have same number of features and same
     # shape
-    if not N.ndim(obs) == N.ndim(code_book):
+    if not np.ndim(obs) == np.ndim(code_book):
         raise ValueError("Observation and code_book should have the same rank")
     elif not d == code_book.shape[1]:
         raise ValueError("Code book(%d) and obs(%d) should have the same " \
@@ -203,7 +254,7 @@ def py_vq(obs, code_book):
     code = zeros(n, dtype=int)
     min_dist = zeros(n)
     for i in range(n):
-        dist = N.sum((obs[i] - code_book) ** 2, 1)
+        dist = np.sum((obs[i] - code_book) ** 2, 1)
         code[i] = argmin(dist)
         min_dist[i] = dist[code[i]]
 
@@ -230,9 +281,9 @@ def _py_vq_1d(obs, code_book):
     raise RuntimeError("_py_vq_1d buggy, do not use rank 1 arrays for now")
     n = obs.size
     nc = code_book.size
-    dist = N.zeros((n, nc))
+    dist = np.zeros((n, nc))
     for i in range(nc):
-        dist[:, i] = N.sum(obs - code_book[i])
+        dist[:, i] = np.sum(obs - code_book[i])
     print dist
     code = argmin(dist)
     min_dist = dist[code]
@@ -253,9 +304,10 @@ def py_vq2(obs, code_book):
             features (eg columns) than obs.
 
     :Note:
-        This could be faster when number of codebooks is small, but it becomes
-        a real memory hog when codebook is large.  It requires NxMxO storage
-        where N=number of obs, M = number of features, and O = number of codes.
+        This could be faster when number of codebooks is small, but it
+        becomes a real memory hog when codebook is large. It requires
+        N by M by O storage where N=number of obs, M = number of
+        features, and O = number of codes.
 
     :Returns:
         code : ndarray
@@ -275,7 +327,7 @@ def py_vq2(obs, code_book):
             number of features (eg columns)""" % (code_book.shape[1], d))
 
     diff = obs[newaxis, :, :] - code_book[:,newaxis,:]
-    dist = sqrt(N.sum(diff * diff, -1))
+    dist = sqrt(np.sum(diff * diff, -1))
     code = argmin(dist, 0)
     min_dist = minimum.reduce(dist, 0) #the next line I think is equivalent
                                       #  - and should be faster
@@ -284,7 +336,7 @@ def py_vq2(obs, code_book):
     return code, min_dist
 
 def _kmeans(obs, guess, thresh=1e-5):
-    """ "raw" version of kmeans.
+    """ "raw" version of k-means.
 
     :Returns:
         code_book :
@@ -294,7 +346,7 @@ def _kmeans(obs, guess, thresh=1e-5):
             Lower means the code_book matches the data better.
 
     :SeeAlso:
-        - kmeans : wrapper around kmeans
+        - kmeans : wrapper around k-means
 
     XXX should have an axis variable here.
 
@@ -318,10 +370,10 @@ def _kmeans(obs, guess, thresh=1e-5):
     """
 
     code_book = array(guess, copy = True)
-    nc = code_book.shape[0]
     avg_dist = []
     diff = thresh+1.
     while diff > thresh:
+        nc = code_book.shape[0]
         #compute membership and distances between obs and code_book
         obs_code, distort = vq(obs, code_book)
         avg_dist.append(mean(distort, axis=-1))
@@ -341,34 +393,58 @@ def _kmeans(obs, guess, thresh=1e-5):
     return code_book, avg_dist[-1]
 
 def kmeans(obs, k_or_guess, iter=20, thresh=1e-5):
-    """Generate a code book with minimum distortion.
+    """Performs k-means on a set of observation vectors forming k
+       clusters. This yields a code book mapping centroids to codes
+       and vice versa. The k-means algorithm adjusts the centroids
+       until sufficient progress cannot be made, i.e. the change in
+       distortion since the last iteration is less than some
+       threshold.
 
     :Parameters:
         obs : ndarray
-            Each row of the array is an observation.  The columns are the
-            "features" seen during each observation The features must be
-            whitened first using the whiten function or something equivalent.
+            Each row of the M by N array is an observation vector. The
+            columns are the features seen during each observation.
+            The features must be whitened first with the whiten
+            function.
+
         k_or_guess : int or ndarray
-            If integer, it is the number of code book elements.  If a 2D array,
-            the array is used as the intial guess for the code book.  The array
-            should have k rows, and the same number of columns (features) as
-            the obs array.
+            The number of centroids to generate. A code is assigned to
+            each centroid, which is also the row index of the centroid
+            in the code_book matrix generated.
+
+            The initial k centroids are chosen by randomly selecting
+            observations from the observation matrix. Alternatively,
+            passing a k by N array specifies the initial k centroids.
+
         iter : int
-            The number of times to restart the kmeans algorithm with a new
-            initial guess.  If k_or_guess is a 2D array (codebook), this
-            argument is ignored and only 1 iteration is run.
+            The number of times to run k-means, returning the codebook
+            with the lowest distortion. This argument is ignored if
+            initial centroids are specified with an array for the
+            k_or_guess paramter. This parameter does not represent the
+            number of iterations of the k-means algorithm.
+
         thresh : float
-            Terminate each kmeans run when the distortion change from one
-            iteration to the next is less than this value.
+            Terminates the k-means algorithm if the change in
+            distortion since the last k-means iteration is less than
+            thresh.
+
     :Returns:
-        codesbook : ndarray
-            The codes that best fit the observation
+        codebook : ndarray
+            A k by N array of k centroids. The i'th centroid
+            codebook[i] is represented with the code i. The centroids
+            and codes generated represent the lowest distortion seen,
+            not necessarily the globally minimal distortion.
+
         distortion : float
-            The distortion between the observations and the codes.
+           The distortion between the observations passed and the
+           centroids generated.
 
     :SeeAlso:
-        - kmeans2: similar function, but with more options for initialization,
-          and returns label of each observation
+        - kmeans2: a different implementation of k-means clustering
+          with more methods for generating initial centroids but without
+          using a distortion change threshold as a stopping criterion.
+        - whiten: must be called prior to passing an observation matrix
+          to kmeans.
 
     Examples
     --------
@@ -403,13 +479,17 @@ def kmeans(obs, k_or_guess, iter=20, thresh=1e-5):
         raise ValueError, 'iter must be >= to 1.'
     if type(k_or_guess) == type(array([])):
         guess = k_or_guess
+        if guess.size < 1:
+            raise ValueError("Asked for 0 cluster ? initial book was %s" % \
+                             guess)
         result = _kmeans(obs, guess, thresh = thresh)
     else:
         #initialize best distance value to a large value
         best_dist = 100000
         No = obs.shape[0]
         k = k_or_guess
-        #print 'kmeans iter: ',
+        if k < 1:
+            raise ValueError("Asked for 0 cluster ? ")
         for i in range(iter):
             #the intial code book is randomly selected from observations
             guess = take(obs, randint(0, No, k), 0)
@@ -440,7 +520,7 @@ def _kpoints(data, k):
     else:
         n = data.size
 
-    p = N.random.permutation(n)
+    p = np.random.permutation(n)
     x = data[p[:k], :].copy()
 
     return x
@@ -460,15 +540,28 @@ def _krandinit(data, k):
             Number of samples to generate.
 
     """
-    mu  = N.mean(data, 0)
-    cov = N.atleast_2d(N.cov(data, rowvar = 0))
+    def init_rank1(data):
+        mu  = np.mean(data)
+        cov = np.cov(data)
+        x = np.random.randn(k)
+        x *= np.sqrt(cov)
+        x += mu
+        return x
+    def init_rankn(data):
+        mu  = np.mean(data, 0)
+        cov = np.atleast_2d(np.cov(data, rowvar = 0))
 
-    # k rows, d cols (one row = one obs)
-    # Generate k sample of a random variable ~ Gaussian(mu, cov)
-    x = N.random.randn(k, mu.size)
-    x = N.dot(x, N.linalg.cholesky(cov).T) + mu
+        # k rows, d cols (one row = one obs)
+        # Generate k sample of a random variable ~ Gaussian(mu, cov)
+        x = np.random.randn(k, mu.size)
+        x = np.dot(x, np.linalg.cholesky(cov).T) + mu
+        return x
 
-    return x
+    nd = np.ndim(data)
+    if nd == 1:
+        return init_rank1(data)
+    else:
+        return init_rankn(data)
 
 _valid_init_meth = {'random': _krandinit, 'points': _kpoints}
 
@@ -486,51 +579,56 @@ _valid_miss_meth = {'warn': _missing_warn, 'raise': _missing_raise}
 
 def kmeans2(data, k, iter = 10, thresh = 1e-5, minit = 'random',
         missing = 'warn'):
-    """Classify a set of points into k clusters using kmean algorithm.
+    """Classify a set of observations into k clusters using the k-means
+       algorithm.
 
-    The algorithm works by minimizing the euclidian distance between data points
-    of cluster means. This version is more complete than kmean (has several
-    initialisation methods).
+    The algorithm attempts to minimize the Euclidian distance between
+    observations and centroids. Several initialization methods are
+    included.
 
     :Parameters:
         data : ndarray
-            Expect a rank 1 or 2 array. Rank 1 are assumed to describe one
-            dimensional data, rank 2 multidimensional data, in which case one
-            row is one observation.
+            A M by N array of M observations in N dimensions or a length
+            M array of M one-dimensional observations.
         k : int or ndarray
-            Number of clusters. If minit arg is 'matrix', or if a ndarray is
-            given instead, it is interpreted as initial cluster to use instead.
-        niter : int
-            Number of iterations to run.
+            The number of clusters to form as well as the number of
+            centroids to generate. If minit initialization string is
+            'matrix', or if a ndarray is given instead, it is
+            interpreted as initial cluster to use instead.
+        iter : int
+            Number of iterations of the k-means algrithm to run. Note
+            that this differs in meaning from the iters parameter to
+            the kmeans function.
         thresh : float
             (not used yet).
         minit : string
-            Method for initialization. Available methods are random, points and
-            uniform:
+            Method for initialization. Available methods are 'random',
+            'points', 'uniform', and 'matrix':
 
-            random uses k points drawn from a Gaussian random generator which
-            mean and variances are estimated from the data.
+            'random': generate k centroids from a Gaussian with mean and
+            variance estimated from the data.
 
-            points choses k points at random from the points in data.
+            'points': choose k observations (rows) at random from data for
+            the initial centroids.
 
-            uniform choses k points from the data such are they form a uniform
-            grid od the dataset (not supported yet).
+            'uniform': generate k observations from the data from a uniform
+            distribution defined by the data set (unsupported).
 
-            matrix means that k has to be interpreted as initial clusters
-            (format is the same than data).
+            'matrix': interpret the k parameter as a k by M (or length k
+            array for one-dimensional data) array of initial centroids.
 
     :Returns:
-        clusters : ndarray
-            the found clusters (one cluster per row).
+        centroid : ndarray
+            A k by N array of centroids found at the last iteration of
+            k-means.
         label : ndarray
-            label[i] gives the label of the ith obversation, that its centroid is
-            cluster[label[i]].
-
+            label[i] is the code or index of the centroid the
+            i'th observation is closest to.
     """
     if missing not in _valid_miss_meth.keys():
         raise ValueError("Unkown missing method: %s" % str(missing))
     # If data is rank 1, then we have 1 dimension problem.
-    nd  = N.ndim(data)
+    nd  = np.ndim(data)
     if nd == 1:
         d = 1
         #raise ValueError("Input of rank 1 not supported yet")
@@ -539,10 +637,13 @@ def kmeans2(data, k, iter = 10, thresh = 1e-5, minit = 'random',
     else:
         raise ValueError("Input of rank > 2 not supported")
 
+    if np.size(data) < 1:
+        raise ValueError("Input has 0 items.")
+
     # If k is not a single value, then it should be compatible with data's
     # shape
-    if N.size(k) > 1 or minit == 'matrix':
-        if not nd == N.ndim(k):
+    if np.size(k) > 1 or minit == 'matrix':
+        if not nd == np.ndim(k):
             raise ValueError("k is not an int and has not same rank than data")
         if d == 1:
             nc = len(k)
@@ -553,7 +654,14 @@ def kmeans2(data, k, iter = 10, thresh = 1e-5, minit = 'random',
                         data")
         clusters = k.copy()
     else:
-        nc = int(k)
+        try:
+            nc = int(k)
+        except TypeError:
+            raise ValueError("k (%s) could not be converted to an integer " % str(k))
+
+        if nc < 1:
+            raise ValueError("kmeans2 for 0 clusters ? (k was %s)" % str(k))
+
         if not nc == k:
             warnings.warn("k was not an integer, was converted.")
         try:
@@ -568,16 +676,16 @@ def kmeans2(data, k, iter = 10, thresh = 1e-5, minit = 'random',
 def _kmeans2(data, code, niter, nc, missing):
     """ "raw" version of kmeans2. Do not use directly.
 
-    Run kmeans with a given initial codebook.  """
+    Run k-means with a given initial codebook.  """
     for i in range(niter):
         # Compute the nearest neighbour for each obs
         # using the current code book
         label = vq(data, code)[0]
         # Update the code by computing centroids using the new code book
         for j in range(nc):
-            mbs = N.where(label==j)
+            mbs = np.where(label==j)
             if mbs[0].size > 0:
-                code[j] = N.mean(data[mbs], axis=0)
+                code[j] = np.mean(data[mbs], axis=0)
             else:
                 missing()
 
@@ -586,13 +694,13 @@ def _kmeans2(data, code, niter, nc, missing):
 if __name__  == '__main__':
     pass
     #import _vq
-    #a = N.random.randn(4, 2)
-    #b = N.random.randn(2, 2)
+    #a = np.random.randn(4, 2)
+    #b = np.random.randn(2, 2)
 
     #print _vq.vq(a, b)
-    #print _vq.vq(N.array([[1], [2], [3], [4], [5], [6.]]), 
-    #        N.array([[2.], [5.]]))
-    #print _vq.vq(N.array([1, 2, 3, 4, 5, 6.]), N.array([2., 5.]))
-    #_vq.vq(a.astype(N.float32), b.astype(N.float32))
-    #_vq.vq(a, b.astype(N.float32))
+    #print _vq.vq(np.array([[1], [2], [3], [4], [5], [6.]]),
+    #             np.array([[2.], [5.]]))
+    #print _vq.vq(np.array([1, 2, 3, 4, 5, 6.]), np.array([2., 5.]))
+    #_vq.vq(a.astype(np.float32), b.astype(np.float32))
+    #_vq.vq(a, b.astype(np.float32))
     #_vq.vq([0], b)
