@@ -12,15 +12,21 @@ def configuration(parent_package='',top_path=None):
                       )
 
     config.add_extension('_fitpack',
-                         sources=['_fitpackmodule.c'],
+                         sources=['src/_fitpackmodule.c'],
                          libraries=['fitpack'],
+                         depends = ['src/__fitpack.h','src/multipack.h']
                         )
 
     config.add_extension('dfitpack',
-                         sources=['fitpack.pyf'],
+                         sources=['src/fitpack.pyf'],
                          libraries=['fitpack'],
                         )
 
+    config.add_extension('_interpolate',
+                         sources=['src/_interpolate.cpp'],
+                         include_dirs = ['src'],
+                         depends = ['src/interpolate.h'])
+
     config.add_data_dir('tests')
 
     return config
