@@ -8,7 +8,6 @@ fmin_coblya(func, x0, cons, args=(), consargs=None, rhobeg=1.0, rhoend=1e-4,
 
 """
 
-from __future__ import nested_scopes
 import _cobyla
 from numpy import copy
 def fmin_cobyla(func, x0, cons, args=(), consargs=None, rhobeg=1.0, rhoend=1e-4,
@@ -47,22 +46,24 @@ def fmin_cobyla(func, x0, cons, args=(), consargs=None, rhobeg=1.0, rhoend=1e-4,
 
     See also:
 
-	fmin, fmin_powell, fmin_cg, 
-	      fmin_bfgs, fmin_ncg -- multivariate local optimizers
-	leastsq -- nonlinear least squares minimizer
+        scikits.openopt, which offers a unified syntax to call this and other solvers
 
-	fmin_l_bfgs_b, fmin_tnc, 
-	      fmin_cobyla -- constrained multivariate optimizers
+        fmin, fmin_powell, fmin_cg,
+              fmin_bfgs, fmin_ncg -- multivariate local optimizers
+        leastsq -- nonlinear least squares minimizer
 
-	anneal, brute -- global optimizers
+        fmin_l_bfgs_b, fmin_tnc,
+              fmin_cobyla -- constrained multivariate optimizers
 
-	fminbound, brent, golden, bracket -- local scalar minimizers
+        anneal, brute -- global optimizers
 
-	fsolve -- n-dimenstional root-finding
+        fminbound, brent, golden, bracket -- local scalar minimizers
 
-	brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding
+        fsolve -- n-dimenstional root-finding
 
-	fixed_point -- scalar fixed-point finder
+        brentq, brenth, ridder, bisect, newton -- one-dimensional root-finding
+
+        fixed_point -- scalar fixed-point finder
 
     """
     err = "cons must be a sequence of callable functions or a single"\
