@@ -1,4 +1,5 @@
 #this program corresponds to special.py
+from decimal import Decimal
 
 from numpy.testing import *
 
@@ -9,6 +10,20 @@ from scipy.signal import lfilter
 from numpy import array, arange
 import numpy as np
 
+# Use this to test for object arrays filtering - numpy 1.2
+# assert_array_almost_equal does not handle object arrays
+def assert_array_almost_equal(x, y, decimal=6, err_msg='', verbose=True):
+    from numpy.core import around, number, float_
+    from numpy.lib import issubdtype
+    from numpy.testing.utils import assert_array_compare
+    def compare(x, y):
+        z = abs(x-y)
+        if not issubdtype(z.dtype, number):
+            z = z.astype(float_) # handle object arrays
+        return around(z, decimal) <= 10.0**(-decimal)
+    assert_array_compare(compare, x, y, err_msg=err_msg, verbose=verbose,
+                         header='Arrays are not almost equal')
+
 class TestConvolve(TestCase):
     def test_basic(self):
         a = [3,4,5,6,5,4]
@@ -121,37 +136,38 @@ class TestChebWin:
         cheb_even = signal.chebwin(54, at=-40)
         assert_array_almost_equal(cheb_even, cheb_even_true, decimal=4)
 
-class TestLinearFilter(TestCase):
+class _TestLinearFilter(TestCase):
+    dt = None
     def test_rank1(self):
-        x = np.linspace(0, 5, 6)
-        b = np.array([1, -1])
-        a = np.array([0.5, -0.5])
+        x = np.linspace(0, 5, 6).astype(self.dt)
+        b = np.array([1, -1]).astype(self.dt)
+        a = np.array([0.5, -0.5]).astype(self.dt)
 
         # Test simple IIR
-        y_r = np.array([0, 2, 4, 6, 8, 10.])
+        y_r = np.array([0, 2, 4, 6, 8, 10.]).astype(self.dt)
         assert_array_almost_equal(lfilter(b, a, x), y_r)
 
         # Test simple FIR
-        b = np.array([1, 1])
-        a = np.array([1])
-        y_r = np.array([0, 1, 3, 5, 7, 9.])
+        b = np.array([1, 1]).astype(self.dt)
+        a = np.array([1]).astype(self.dt)
+        y_r = np.array([0, 1, 3, 5, 7, 9.]).astype(self.dt)
         assert_array_almost_equal(lfilter(b, a, x), y_r)
 
         # Test IIR with initial conditions
-        b = np.array([1, 1])
-        a = np.array([1])
-        zi = np.array([1])
-        y_r = np.array([1, 1, 3, 5, 7, 9.])
-        zf_r = np.array([5])
+        b = np.array([1, 1]).astype(self.dt)
+        a = np.array([1]).astype(self.dt)
+        zi = np.array([1]).astype(self.dt)
+        y_r = np.array([1, 1, 3, 5, 7, 9.]).astype(self.dt)
+        zf_r = np.array([5]).astype(self.dt)
         y, zf = lfilter(b, a, x, zi=zi)
         assert_array_almost_equal(y, y_r)
         assert_array_almost_equal(zf, zf_r)
 
-        b = np.array([1, 1, 1])
-        a = np.array([1])
-        zi = np.array([1, 1])
-        y_r = np.array([1, 2, 3, 6, 9, 12.])
-        zf_r = np.array([9, 5])
+        b = np.array([1, 1, 1]).astype(self.dt)
+        a = np.array([1]).astype(self.dt)
+        zi = np.array([1, 1]).astype(self.dt)
+        y_r = np.array([1, 2, 3, 6, 9, 12.]).astype(self.dt)
+        zf_r = np.array([9, 5]).astype(self.dt)
         y, zf = lfilter(b, a, x, zi=zi)
         assert_array_almost_equal(y, y_r)
         assert_array_almost_equal(zf, zf_r)
@@ -159,13 +175,16 @@ class TestLinearFilter(TestCase):
     def test_rank2(self):
         shape = (4, 3)
         x = np.linspace(0, np.prod(shape) - 1, np.prod(shape)).reshape(shape)
+        x = x.astype(self.dt)
 
-        b = np.array([1, -1])
-        a = np.array([0.5, 0.5])
+        b = np.array([1, -1]).astype(self.dt)
+        a = np.array([0.5, 0.5]).astype(self.dt)
+
+        y_r2_a0 = np.array([[0, 2, 4], [6, 4, 2], [0, 2, 4], [6 ,4 ,2]],
+                           dtype=self.dt)
 
-        y_r2_a0 = np.array([[0, 2, 4], [6, 4, 2], [0, 2, 4], [6 ,4 ,2]])
         y_r2_a1 = np.array([[0, 2, 0], [6, -4, 6], [12, -10, 12],
-                            [18, -16, 18]])
+                            [18, -16, 18]], dtype=self.dt)
 
         y = lfilter(b, a, x, axis = 0)
         assert_array_almost_equal(y_r2_a0, y)
@@ -177,42 +196,43 @@ class TestLinearFilter(TestCase):
         # Test initial condition handling along axis 1
         shape = (4, 3)
         x = np.linspace(0, np.prod(shape) - 1, np.prod(shape)).reshape(shape)
+        x = x.astype(self.dt)
 
-        b = np.array([1, -1])
-        a = np.array([0.5, 0.5])
+        b = np.array([1, -1]).astype(self.dt)
+        a = np.array([0.5, 0.5]).astype(self.dt)
 
         y_r2_a0_1 = np.array([[1, 1, 1], [7, -5, 7], [13, -11, 13],
-                              [19, -17, 19]])
-        zf_r = np.array([-5, -17, -29, -41])[:, np.newaxis]
+                              [19, -17, 19]], dtype=self.dt)
+        zf_r = np.array([-5, -17, -29, -41])[:, np.newaxis].astype(self.dt)
         y, zf = lfilter(b, a, x, axis = 1, zi = np.ones((4, 1)))
         assert_array_almost_equal(y_r2_a0_1, y)
         assert_array_almost_equal(zf, zf_r)
 
-    # Disabled because it crashes lfilter for now
     #@dec.skipif(True, "Skipping lfilter test with initial condition along "\
     #                  "axis 0: it segfaults ATM")
     def test_rank2_init_cond_a0(self):
         # Test initial condition handling along axis 0
         shape = (4, 3)
         x = np.linspace(0, np.prod(shape) - 1, np.prod(shape)).reshape(shape)
+        x = x.astype(self.dt)
 
-        b = np.array([1, -1])
-        a = np.array([0.5, 0.5])
+        b = np.array([1, -1]).astype(self.dt)
+        a = np.array([0.5, 0.5]).astype(self.dt)
 
-        y_r2_a0_0 = np.array([[1, 3, 5], [5, 3, 1], [1, 3, 5], [5 ,3 ,1]])
-        zf_r = np.array([[-23, -23, -23]])
+        y_r2_a0_0 = np.array([[1, 3, 5], [5, 3, 1], [1, 3, 5], [5 ,3 ,1]], 
+                             dtype=self.dt)
+        zf_r = np.array([[-23, -23, -23]], dtype=self.dt)
         y, zf = lfilter(b, a, x, axis = 0, zi = np.ones((1, 3)))
         assert_array_almost_equal(y_r2_a0_0, y)
         assert_array_almost_equal(zf, zf_r)
 
-    # Disabled because it crashes lfilter for now
     #@dec.skipif(True, "Skipping rank > 2 test for lfilter because its segfaults ATM")
     def test_rank3(self):
         shape = (4, 3, 2)
         x = np.linspace(0, np.prod(shape) - 1, np.prod(shape)).reshape(shape)
 
-        b = np.array([1, -1])
-        a = np.array([0.5, 0.5])
+        b = np.array([1, -1]).astype(self.dt)
+        a = np.array([0.5, 0.5]).astype(self.dt)
 
         # Test last axis
         y = lfilter(b, a, x)
@@ -220,5 +240,28 @@ class TestLinearFilter(TestCase):
             for j in range(x.shape[1]):
                 assert_array_almost_equal(y[i, j], lfilter(b, a, x[i, j]))
 
+    def test_empty_zi(self):
+        """Regression test for #880: empty array for zi crashes."""
+        a = np.ones(1).astype(self.dt)
+        b = np.ones(1).astype(self.dt)
+        x = np.arange(5).astype(self.dt)
+        zi = np.ones(0).astype(self.dt)
+        lfilter(b, a, x, zi=zi)
+
+class TestLinearFilterFloat32(_TestLinearFilter):
+    dt = np.float32
+
+class TestLinearFilterFloat64(_TestLinearFilter):
+    dt = np.float64
+
+class TestLinearFilterComplex64(_TestLinearFilter):
+    dt = np.complex64
+
+class TestLinearFilterComplex128(_TestLinearFilter):
+    dt = np.complex128
+
+class TestLinearFilterDecimal(_TestLinearFilter):
+    dt = np.dtype(Decimal)
+
 if __name__ == "__main__":
     run_module_suite()
