@@ -49,7 +49,7 @@ def diric(x,n):
 
 
 def jnjnp_zeros(nt):
-    """Compute nt (<=1400) zeros of the bessel functions Jn and Jn'
+    """Compute nt (<=1200) zeros of the bessel functions Jn and Jn'
     and arange them in order of their magnitudes.
 
     Outputs (all are arrays of length nt):
@@ -81,7 +81,7 @@ def jnyn_zeros(n,nt):
         raise ValueError, "Arguments must be integers."
     if (nt <=0):
         raise ValueError, "nt > 0"
-    return specfun.jyzo(n,nt)
+    return specfun.jyzo(abs(n),nt)
 
 def jn_zeros(n,nt):
     """Compute nt zeros of the Bessel function Jn(x).
@@ -232,7 +232,6 @@ def sph_yn(n,z):
         raise ValueError, "n must be a non-negative integer."
     if (n < 1): n1 = 1
     else: n1 = n
-    print z
     if iscomplex(z) or less(z,0):
         nm,jn,jnp,yn,ynp = specfun.csphjy(n1,z)
     else:
@@ -395,12 +394,6 @@ def fresnel_zeros(nt):
         raise ValueError, "Argument must be positive scalar integer."
     return specfun.fcszo(2,nt), specfun.fcszo(1,nt)
 
-def gammaincinv(a,y):
-    """returns the inverse of the incomplete gamma integral in that it
-    finds x such that gammainc(a,x)=y
-    """
-    return gammainccinv(a,1-y)
-
 def hyp0f1(v,z):
     """Confluent hypergeometric limit function 0F1.
     Limit as q->infinity of 1F1(q;a;z/q)
@@ -536,9 +529,11 @@ def lqmn(m,n,z):
         raise ValueError, "z must be scalar."
     m = int(m)
     n = int(n)
-    if (m*n == 0):
-        mm = max(1,m)
-        nn = max(1,n)
+
+    # Ensure neither m nor n == 0
+    mm = max(1,m)
+    nn = max(1,n)
+
     if iscomplex(z):
         q,qd = specfun.clqmn(mm,nn,z)
     else:
