@@ -18,5 +18,5 @@ try:
 except:
     pass
 
-from numpy.testing import NumpyTest
-test = NumpyTest().test
+from numpy.testing import Tester
+test = Tester().test
