@@ -310,7 +310,7 @@ num_to_c_types['h'] = 'short'
 num_to_c_types['H'] = 'npy_ushort'
 num_to_c_types['i'] = 'int'
 num_to_c_types['I'] = 'npy_uint'
-
+num_to_c_types['?'] = 'bool'
 num_to_c_types['l'] = 'long'
 num_to_c_types['L'] = 'npy_ulong'
 
@@ -455,10 +455,6 @@ class catchall_converter(scxx_converter):
     def type_match(self,value):
         return 1
 
-def test(level=10,verbosity=1):
-     from numpy.testing import NumpyTest
-     NumpyTest().test(level, verbosity)
-
 if __name__ == "__main__":
     x = list_converter().type_spec("x",1)
     print x.py_to_c_code()
@@ -468,4 +464,3 @@ if __name__ == "__main__":
     print x.declaration_code(inline=1)
     print
     print x.cleanup_code()
-
